% Function to integrate a system of first-order 
% initial-value ODE's by the backward modified Euler method.
% 
% One Backward Euler step for the equation dy/dt = f(t,y) reads
% 
% y_n+1 = y_n + h * f(t_n+1, y_n+1)
% 
% The integrator accepts the same arguments as the built-in
% Matlab solvers ode23,.  The time step needs to be set
% by the options argument, field InitialStep.
% 
% Arguments:
%    rhsfun  = right-hand side function: returns f(t,y) 
%              as a column vector.
%    tspan   = time span (start time, usually zero, 
%              and stop time).
%    y0      = initial condition.
%    h       = step size; constant.
%
%
% Copyright (C) 2007, Petr Krysl
% 
function [ts, ys] = odeimeul2nd (rhsfun, tspan, y0, options, varargin)
    h = options.InitialStep(1);
    if ~isfield( options,'RelTol')
        options.RelTol =1e-3;
    else
        if isempty(options.RelTol)
            options.RelTol =1e-3;
        end
    end
    K=options.K;
    M=options.M;
    maxit=12;
    if isfield( options,'maxit')
        maxit=options.maxit;
    end
    nsteps = ceil ((tspan(2) - tspan(1)) / h);
    ts = zeros(nsteps+1,1);
    ys = zeros(nsteps+1,length(y0));
    n=length(y0)/2;
    t = tspan(1);
    ts(1) = t;
    ys(1,:) = y0';
    for step = 2:nsteps+1
        if (ts(step-1) + h >tspan(2)), h=tspan(2)-ts(step-1); end
        x0=ys(step-1,1:n)';
        v0=ys(step-1,n+1:end)';
        v1=v0;
        x1=x0+h*v0;
        B= [(M-(h^2)/2*K),zeros(size(M));zeros(size(M)),(M-(h^2)/2*K)];
        C = [zeros(size(M)),h*M;-h*K,zeros(size(M))];
        [V,D]=eig(B\C);
        plot(abs(diag(D)))
        for it=1:maxit
            x1 =(M-(h^2)/2*K)\(h*M*v1+M*x0);
            v1 =(M-(h^2)/2*K)\(M*v0-h*K*x1);
        end
%         x1=(4*M^2+h^4*K^2)\(-2*(-2*M+h^2*K)*M*x0+4*h*M^2*v0);
%         v1= (4*M^2+h^4*K^2)\(-4*h*M*K*x0-2*(-2*M+h^2*K)*M*v0);
        ts(step) = ts(step-1) + h;
        ys(step,:) = [x1',v1'];
    end
    return;
end
