% Compute the electrical conductivity matrices of the individual gcells.
%
% function ems = electrical_conductivity(self, geom, phi, temp)
%
% Return an array of the element matrices so they may be assembled.
%    Here
%            geom=geometry field
%            phi=electrical potential field
%            temp=temperature field
%
% Note: The electrical conductivity may depend on the temperature.
function ems = electrical_conductivity(self, geom, phi, temp)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coor
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material
    mat = get(self.feblock, 'mater');
    sigma = get(get(mat,'property'),'electrical_conductivity');
    if (strcmp(class(sigma),'function_handle'))
        need_to_eval=1;
    else
        need_to_eval=0; Sigma = sigma;
    end
    % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(phi,conns(i,:),'eqnums');
        Ke{i} =zeros(nfens);
    end
    xs =get(geom,'values');% retrieve the geometry information
   % Now loop over all gcells in the block
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        if (~ isempty(temp)), T = gather(temp,conn,'values','noreshape');% node coord
        end
        % Loop over all integration points
        for j=1:npts_per_gcell
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            Ndersp = Nders{j}/J;
            Rm = material_directions(self,gcells,pc(j,:),x);
            if (need_to_eval)
                Sigma = sigma (Ns{j}'*T);
            end
            if (~isempty(Rm))
                Sig=Rm*Sigma*Rm';%  wrt local coor
            else
                Sig=Sigma;% no transformation
            end
            Ke{i} = Ke{i} + Ndersp*(Sig*(Jac*w(j)))*Ndersp' ;
        end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end
