% Compute the thermal conductivity matrices of the individual gcells. 
%
% function ems = conductivity(self, geom, temp)
%
% Return an array of the element matrices so they may be assembled.
%    Call as     ems = conductivity(feb, geom, temp)
%    where   geom=geometry field
%            temp=temperature field
% 
% The thermal conductivity may depend on the temperature.  The thermal
% property would define the thermal conductivity as a function of the
% temperature, which is then evaluated at each quadrature point.
% 
function ems = conductivity(self, geom, temp)
    gcells = get(self.feblock_diffusion,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self.feblock_diffusion, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self.feblock_diffusion, 'mater');
    kappa = get(get(mat,'property'),'thermal_conductivity');
    if (strcmp(class(kappa),'function_handle'))
        need_to_eval=1;
    else
        need_to_eval=0; KAPPA = kappa;
    end
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    eqnumss =gather(temp,(1:get(temp,'nfens')),'eqnums','noreshape');
    for i=1:size(conns,1)
        eqnums{i} =eqnumss(conns(i,:));
        Ke{i} =zeros(nfens);
    end
    xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    Ts =gather(temp,(1:get(geom,'nfens')),'values','noreshape');
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x = xs(conn, :); % coord
        T = Ts(conn, :);% node temperatures
        % Loop over all integration points
        for j=1:npts_per_gcell
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            Ndersp = Nders{j}/J;
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(Ns{j}'*x,J);
            end
            if (need_to_eval)
                KAPPA = kappa (Ns{j}'*T);
            end
            if (~isempty(Rm))
                KAPPA=Rm*KAPPA*Rm';%  wrt local coor
            end
            Ke{i} = Ke{i} + Ndersp* (KAPPA* (Jac * w(j)))*Ndersp' ;
        end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end
