% feblock
%   This tree of classes provides operations on block of finite elements.
% 
% 
% - feblock 
%   This is the base class, from which specialized blocks may
%   be derived to implement or override behaviour.
%
%   @feblock/feblock - Constructor.
%   @feblock/draw - Draw the cells.
%   @feblock/draw_mesh - Draw the mesh.
%   @feblock/draw_mesh_quality - Draw mesh quality.
%   @feblock/draw_isosurface - Draw an iso-surface.
%   @feblock/measure - Integrate simple function over the block cells.
%   @feblock/measure_field - Integrate field over the block cells.
%   @feblock/get - Get an attribute
%   @feblock/set - Set an attribute.
%   @feblock/material_directions - Compute orientation matrix.
%
% 
% - feblock_defor_ss 
%   This class implements the small strain, small
%   displacement deformation behavior.
%
%   @feblock_defor_ss/feblock_defor_ss -  Constructor.
%   @feblock_defor_ss/stiffness - Compute element stiffness matrices.
%   @feblock_defor_ss/nz_ebc_loads - Compute element vectors
%        of loads due to nonzero boundary conditions.
%   @feblock_defor_ss/distrib_loads - Compute element vectors
%        of distributed loads.
%   @feblock_defor_ss/thermal_strain_loads - Compute element vectors
%        of thermal strain loads.
%   @feblock_defor_ss/mass - Compute element  mass  matrices.
%   @feblock_defor_ss/flux_L2_error - Compute elementwise  L2 error.
%   @feblock_defor_ss/acoustic_coupling - Compute acoustic-structural 
%        coupling matrices.
%   @feblock_defor_ss/infsup_Gh - Compute element matrices for the inf-sup test.
%   @feblock_defor_ss/infsup_Sh - Compute element matrices for the inf-sup
%   test.
%   @feblock_defor_ss/field_from_integration_points - Create a field from
%        quantities at integration points.
%   @feblock_defor_ss/field_from_integration_points_spr - Create a field from
%        quantities at integration points using super-convergent patch recovery.
%   @feblock_defor_ss/draw_integration_points - Draw quantities at
%                                       integration points.
%   @feblock_defor_ss/get - Gets an attribute.
%   @feblock_defor_ss/inspect_integration_points - Inspect the integration
%        point quantities.
%   @feblock_defor_ss/set_matstate - Set material states.
% 
% 
% - feblock_defor_ss_sri 
%   This class implements the small strain, small
%   displacement deformation behavior, with selective reduce
%   integration (SRI).  Derived from feblock_defor_ss.
% 
%   @feblock_defor_ss_sri/stiffness - 
%   @feblock_defor_ss_sri/feblock_defor_ss_sri - Constructor.
%   @feblock_defor_ss_sri/get - Get an attribute 
%   @feblock_defor_ss/infsup_Gh - Compute element matrices for the inf-sup test.
%   @feblock_defor_ss/infsup_Sh - Compute element matrices for the inf-sup
%   test.
% 
% 
% - feblock_diffusion 
%   This class implements the behavior for thermal analysis.
% 
%   @feblock_diffusion/feblock_diffusion - Constructor
%   @feblock_diffusion/conductivity - Compute element conductivity matrices.
%   @feblock_diffusion/nz_ebc_loads_conductivity - Compute element vectors
%        of loads due to nonzero boundary conditions,
%        contribution of the conductivity matrix.
%   @feblock_diffusion/capacity - Compute element capacity   matrices.
%   @feblock_diffusion/nz_ebc_loads_capacity - Compute element vectors
%        of loads due to nonzero boundary conditions,
%        contribution of the capacity matrix.
%   @feblock_diffusion/source_loads - Compute element vectors
%        of loads due to internal heat sources and sinks.
%   @feblock_diffusion/flux_loads - Compute element vectors
%        of loads due to boundary flux.
%   @feblock_diffusion/surface_transfer - Compute element
%        surface-transfer conductivity   matrices.
%   @feblock_diffusion/nz_ebc_loads_surface_transfer - Compute element vectors
%        of loads due to nonzero boundary conditions,
%        contribution of the  Surface-transfer matrix.
%   @feblock_diffusion/surface_transfer_loads - Compute element vectors
%        of loads due to surface transfer flux
%   @feblock_diffusion/field_from_integration_points - Create a field from
%        quantities at integration points.
%   @feblock_diffusion/field_from_integration_points_spr - Create a field from
%        quantities at integration points using super-convergent patch recovery.
%   @feblock_diffusion/draw_integration_points - 
%   @feblock_diffusion/get - Get an attribute
%   @feblock_diffusion/flux_L2_error - Compute elementwise  L2 error.
% 
% 
% - feblock_defor_taut_wire 
%   This class implements a toy block, used only as a
%   demonstration in the first part of the finite element book.
% 
%   @feblock_defor_taut_wire/stiffness - Compute element stiffness
%                                       matrices.
%   @feblock_defor_taut_wire/get - Get an attribute
%   @feblock_defor_taut_wire/nz_ebc_loads - Compute element vectors
%                     of loads due to nonzero boundary conditions.
%   @feblock_defor_taut_wire/feblock_defor_taut_wire - Constructor
%   @feblock_defor_taut_wire/mass - Compute element  mass
%                                       matrices.
%   @feblock_defor_taut_wire/body_loads - Compute element vectors
%                     of loads due to  Distributed transversal load.
%
% 
% - feblock_acoustics 
%   This class implements the behavior for linear acoustic analysis.
% 
%   @feblock_acoustics/feblock_acoustics - Constructor
%   @feblock_acoustics/acoustic_mass - Compute element acoustic mass  matrices.
%   @feblock_acoustics/nz_ebc_loads_acoustic_mass - Compute element vectors
%        of loads due to nonzero boundary conditions,
%        contribution of the acoustic mass matrix.
%   @feblock_acoustics/acoustic_stiffness - Compute element acoustic stiffness  matrices.
%   @feblock_acoustics/nz_ebc_loads_acoustic_stiffness - Compute element vectors
%        of loads due to nonzero boundary conditions,
%        contribution of the acoustic stiffness matrix.
%   @feblock_acoustics/pressure_flux_loads - Compute element vectors
%        of loads due to boundary pressure flux.
%   @feblock_acoustics/pressure_to_resultant_force - Compute element
%        rigid body transformation matrices.
%   @feblock_acoustics/pressure_to_resultant_torque - Compute element
%        rigid body transformation matrices.
%   @feblock_acoustics/surface_acoustic_ABC - Compute element matrices for
%        absorbing boundary conditions
%   @feblock_acoustics/field_from_integration_points - Create a field from
%        quantities at integration points.
%   @feblock_acoustics/draw_integration_points - 
%   @feblock_acoustics/get - Get an attribute
% 
% 
% - feblock_electrostatics
%   This class implements the behavior for electrostatic analysis.
% 
%   @feblock_electrostatics/feblock_electrostatics - Constructor
%   @feblock_electrostatics/electrical_conductivity - Compute element 
%        electrical conductivity matrices.
%   @feblock_electrostatics/nz_ebc_loads_electrical_conductivity - Compute element vectors
%        of loads due to nonzero boundary conditions,
%        contribution of the electrical_conductivity matrix.
%   @feblock_electrostatics/resistive_heating_loads - Compute element
%        vectors for resistive (Joule) heating loads.
%   @feblock_electrostatics/draw_integration_points - 
%   @feblock_electrostatics/get - Get an attribute
% 
%
% - feblock_defor_ss_beam3_corot 
%   This class implements the small strain, arbitrarily large
%   displacement, corotational beam element.
%
%   @feblock_defor_ss_beam3_corot/feblock_defor_ss_beam3_corot -  Constructor.
%   @feblock_defor_ss_beam3_corot/draw_integration_points - Draw quantities at
%                                       integration points.
%   @feblock_defor_ss_beam3_corot/stiffness - Compute element stiffness matrices.
%   @feblock_defor_ss_beam3_corot/distrib_loads - Compute element vectors
%        of distributed loads.
%   @feblock_defor_ss_beam3_corot/get - Gets an attribute.
%   @feblock_defor_ss_beam3_corot/inspect_integration_points - Inspect the integration
%        point quantities.
%   @feblock_defor_ss_beam3_corot/nz_ebc_loads - Compute element vectors
%        of loads due to nonzero boundary conditions.
%   @feblock_defor_ss_beam3_corot/field_from_integration_points - Create a field from
%        quantities at integration points.
%   @feblock_defor_ss_beam3_corot/field_from_integration_points_spr - Create a field from
%        quantities at integration points using super-convergent patch recovery.
%   @feblock_defor_ss_beam3_corot/thermal_strain_loads - Compute element vectors
%        of thermal strain loads.
%   @feblock_defor_ss_beam3_corot/mass - Compute element  mass  matrices.
%   @feblock_defor_ss_beam3_corot/set_matstate - Set material states.
%   @feblock_defor_ss_beam3_corot/flux_L2_error - Compute elementwise  L2 error.
%   @feblock_defor_ss_beam3_corot/acoustic_coupling - Compute acoustic-structural 
%        coupling matrices.
%   @feblock_defor_ss_beam3_corot/infsup_Gh - Compute element matrices for the inf-sup test.
%   @feblock_defor_ss_beam3_corot/infsup_Sh - Compute element matrices for the inf-sup
%   test.
% 
% 
% - feblock_defor_ss_nice 
%   This class implements the small strain, small
%   displacement deformation behavior in the format of assumed-strain
%   Nodally-Integrated Continuum Elements (NICE).
%
%   @feblock_defor_ss_nice/feblock_defor_ss_nice -  Constructor.
%   @feblock_defor_ss_nice/draw_integration_points - Draw quantities at
%                                       integration points.
%   @feblock_defor_ss_nice/stiffness - Compute element stiffness matrices.
%   @feblock_defor_ss_nice/distrib_loads - Compute element vectors
%        of distributed loads.
%   @feblock_defor_ss_nice/get - Gets an attribute.
%   @feblock_defor_ss_nice/inspect_integration_points - Inspect the integration
%        point quantities.
%   @feblock_defor_ss_nice/nz_ebc_loads - Compute element vectors
%        of loads due to nonzero boundary conditions.
%   @feblock_defor_ss_nice/field_from_integration_points - Create a field from
%        quantities at integration points.
%   @feblock_defor_ss_nice/field_from_integration_points_spr - Create a field from
%        quantities at integration points using super-convergent patch recovery.
%   @feblock_defor_ss_nice/thermal_strain_loads - Compute element vectors
%        of thermal strain loads.
%   @feblock_defor_ss_nice/mass - Compute element  mass  matrices.
%   @feblock_defor_ss_nice/set_matstate - Set material states.
%   @feblock_defor_ss_nice/flux_L2_error - Compute elementwise  L2 error.
%   @feblock_defor_ss_nice/acoustic_coupling - Compute acoustic-structural 
%        coupling matrices.
%   @feblock_defor_ss_nice/infsup_Gh - Compute element matrices for the inf-sup test.
%   @feblock_defor_ss_nice/infsup_Sh - Compute element matrices for the inf-sup
%   test.
% 
% 
