% Compute the matrices that link a deformable structure in an acoustic 
% fluid to the fluid pressures. The fluid pressures are assumed to be given
% at the centroids of the surface finite elements (panels). On each panel
% the pressure is assumed to be uniform.
%
% Inputs:
% R= radius of the sphere,
% nperradius= number of element edges along the radius of the sphere (must
%      be an even number), 
% Es= Young's modulus, 
% nus= Poisson ratio, 
% rhos= mass density, 
% neigvs= number of eigenvectors to be solved for; only those correspond 
%      to nonzero natural frequencies of vibration are returned, 
%      the rigid body modes are ignored 
% 
% Outputs:
% mE= matrix of eigenvectors, one per column; see equation (20) of [1]. 
% mId= identity matrix, of dimension equal to the number of eigenvectors;
%      see equation (20) of [1]. 
% mOm= diagonal matrix, with squares of eigenvalues on the diagonal; see
%      equation (20) of [1]. 
% mG= acoustic-structural coupling matrix, mG=C*D; see equation (16) of [1].
%      size(mG) is [3*N,Ne], where Ne = number of elements on the surface 
%      model of the sphere, and N = number of nodes in the finite element
%      model (the entire volume mesh)
% mD= matrix of surface finite element areas; see equation (16) of [1]. 
%      size(mD) is [Ne,Ne], where Ne = number of elements on the surface 
% mX= matrix of coordinates of the nodes on the surface of the sphere; 
%      they are ordered by the pressure degree of freedom 
% mConn= matrix of connectivities of the surface finite elements: the jth
%      connects the nodes with coordinates mX(mConn(j,:),:)
% 
% References
% [1] G.W. Benthien and H.A. Schenck, STRUCTURAL-ACOUSTIC COUPLING 
% 
function [mE,mId,mOm,mG,mD,mX,mConn]=...
    sphere_acoustic_coupling_panels(R,nperradius,Es,nus,rhos,neigvs)
    graphics= ~true;% plot the solution as it is computed?
    if (~exist('R'))
        graphics= true;% no arguments, run interactively;plot the solution
        R=500;% radius of the sphere, millimeters
    end
    if (~exist('nperradius'))
        nperradius=10;% number of edges along the radius of the sphere
    end
    if (~exist('Es'))
        Es= 69000;%Aluminum material parameters, [MPa]
    end
    if (~exist('rhos'))
        rhos= 2.73e-9;% Aluminum material parameters, [1000*kg/mm^3]
    end
    if (~exist('nus'))
        nus=0.33;%Aluminum material parameters
    end
    if (~exist('neigvs'))
        neigvs = 40;% number of eigenvectors to use
    end
    
    debug=false;
    
    %Shift to be able to solve for the frequencies of a free-floating
    %sphere; expecting the first frequency to be above 100 Hz
    kappa_T= 2.5;% lowest (torsional) mode
    c_T=sqrt(Es/rhos /2/(1+ nus));
    f=kappa_T*c_T/(2*pi*R);
    OmegaShift=(f/10) ^ 2;
    
    % Eigenvalue solver options
    options.issym= true;
    options.tol=1e-35;
    options.maxit= 500;
    options.disp= 2;
    
    
    tol=R/10000;% geometrical tolerance
    
    % Mesh
    [fens,gcells]=H8_sphere_n(R,nperradius);
    [fens1,gcells1] = mirror_mesh(fens, gcells, [-1,0,0], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
    gcells=cat(gcells1,gcells2);
    [fens1,gcells1] = mirror_mesh(fens, gcells, [0,-1,0], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
    gcells=cat(gcells1,gcells2);
    [fens1,gcells1] = mirror_mesh(fens, gcells, [0,0,-1], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
    gcells=cat(gcells1,gcells2);
    %
    % gv=drawmesh({fens,gcells},'gcells','facecolor','white')
    
    % Renumber the elements to minimize the fill in
    [fens, gcells]=renum_mesh(fens, gcells, 'symrcm');
    
    % Extract the boundary cells 
    boundary_gcells=mesh_bdry(gcells);
    
    % Material
    prop = property_linel_iso (struct('E',Es,'nu',nus,'rho',rhos));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    % Finite element block
    feb = feblock_defor_ss (struct ('mater',mater, 'gcells',gcells,...
        'integration_rule',gauss_rule(3,2),'Rm', []));
    % Geometry
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    limits =reshape([min(get(geom,'values'));max(get(geom,'values'))], 6, 1);
    
     % Define the displacement field
    u   = 0*geom; % zero out
    % No EBC's: free-floating
    u   = apply_ebc (u);
    % Number equations
    u   = numbereqns (u);
    
    % Define the pressure field: it has degrees of freedom at all the nodes of the mesh
    P = field(struct ('name',['P'], 'dim', 1,'nfens',get(geom,'nfens')));
    P = numbereqns (P);
    
    % Find the nodes on the surface
    sn=connected_nodes(boundary_gcells);
    on = setdiff((1:get(geom,'nfens')),sn);
    % Put a constraint pressure==0 at the interior nodes
    P = set_ebc(P, on, true, [], 0);
    P = apply_ebc (P);
    % Only the surface nodes will carry a nonzero equation number
    permutation=zeros(get(geom,'nfens'),1);
    permutation(sn)=(1:length(sn))';
    permutation(on)=0;
    P = numbereqns(P,permutation);
    
    % Assemble the system matrix
    K = start (sparse_sysmat, get(u, 'neqns'));
    evs=stiffness(feb, geom, u);
    K = assemble (K, evs);
    M = start (sparse_sysmat, get(u, 'neqns'));
    ems=mass(feb, geom, u);
    % If lumped mass matrix, uncomment this block
    % mat=get(ems,'mat');
    % for j=1:length(mat)% Hinton, Rock, Zienkiewicz lumping
    %     Me=mat{j};
    %     em2=sum(sum(Me));
    %     dem2=sum(diag(Me));
    %     mat{j}=diag(diag(Me)/dem2*em2);
    % end
    % ems=set(ems,'mat',mat);
    % % Consistent or lumped mass matrix
    M = assemble (M, ems);
    disp(['Number of DOF =' num2str(get(u, 'neqns'))]);
    
    % Make sure the mass and stiffness matrices are symmetric
    mK=get(K,'mat');
    mK=mK/2+mK'/2;
    mM=get(M,'mat');
    mM=mM/2+mM'/2;
    %
    % Adjust the number of eigenvalues to be solved for the account for the rigid body modes
    [W,Omega]=eigs(mK+OmegaShift*mM,mM,neigvs+6,'SM', options);
    % Undo the shift 
    Omega =diag(diag(Omega)-OmegaShift);
    % Sort the eigenvalues and eigenvectors, and remove the rigid body modes
    [Ignore,ix] =sort (diag(Omega));
    Omegas=diag(Omega(ix(7:end),ix(7:end)));
    W=W(:,ix(7:end));
    clear Omega ix
    
    for i=1:length(Omegas)
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omegas(i))/2/pi) ' [Hz]']);
    end
    
    
    % Plot
    if graphics
        Camera =1.0e+003 * [-1.0133   -0.8612    1.1200   -0.0296   -0.0210    0.0345         0         0    0.0010    0.0333];
        xyz=get(geom,'values');
        box =reshape([min(xyz);max(xyz)],1,6);
        annloc=[mean(box(1:2))-R/2, min(box(3:4))+0, -R/2];
        bg=mesh_bdry(gcells);
        bfeb = feblock(struct ('gcells',bg));
        gv=graphic_viewer;
        gv=reset (gv,struct('limits',  limits));
        set_graphics_defaults
        w=clone(u,'w'); % make a copy of u
        % Take a snapshot of the mesh
        % First a static display
        gv=reset (gv,[]);
        camset(gv,Camera);
        draw(bfeb,gv, struct ('x', geom, 'u', 0*w,'facecolor','b','edgecolor',[1,1,1]*0.7, 'linewidth',2));
        axis off
        draw_axes (gv,struct('length',R/3,'origin', [-R,+R,-R]));
        pause(1);
        saveas(gcf,[mfilename '-mesh.png']);
        %
        scale=R*10/2;
        % compute normals to the surface
        normals=get(geom,'values');
        for j=1:size(normals, 1)
            nn=norm(normals(j,:));
            if (nn>0)
                normals(j,:) = normals(j,:)'/nn;
            else
                normals(j,:) = [0,0,0];
            end
        end
        wmagmax=0;
        for i=1:length(Omegas)
            w = scatter_sysvec(w, W(:,i));
            w = w*(1/norm(w ));
            wv = get(w,'values');
            wmag =zeros(size(wv, 1),1 );
            for j=1:size(wv, 1)
                wmag(j) = wv(j,:)*normals(j,:)';
            end 
            wmagmax=max( [wmagmax, max(wmag )] );
        end
        for i=1:length(Omegas)
            disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omegas(i))/2/pi) ' [Hz]']);
            w = scatter_sysvec(w, W(:,i));
            w = w*(1/norm(w ));
            wv = get(w,'values');
            wmag =zeros(size(wv, 1),1 );
            for j=1:size(wv, 1)
                wmag(j) = abs(wv(j,:)*normals(j,:)');
            end 
            wmagf = field(struct ('name',['wmagf'], 'dim', 1, 'data',wmag));
            dcm=data_colormap(struct ('range',[0,+wmagmax], 'colormap',jet));
            colors=map_data(dcm, wmag);
            colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
            % First a static display
            gv=reset (gv,[]);
                camset(gv,Camera);
            draw(bfeb,gv, struct ('x', geom, 'u', scale*w,'colorfield',colorfield,'edgecolor',[1,1,1]*0.7, 'linewidth',2));
            draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omegas(i))/2/pi) ' [Hz]'],...
                struct('fontsize', 18, 'fontname', 'Times'));
            axis off
            draw_axes (gv,struct('length',R/3,'origin', [-R,+R,-R]));
            draw_colorbar(gv,struct('colormap',get(dcm,'colormap'),...
                'position',[0.87, 0.24, 0.05, 0.5],...
                'minmax', get(dcm,'range')/wmagmax,...
                'label','$|u_n|$', 'fontname', 'Times', 'interpreter', 'latex'));
            pause(1);
            saveas(gcf,[mfilename '-m' num2str(i) '.png']);
            % Now a dynamic one
            for xscale=scale*sin((0:1:42)/21*2*pi)
                gv=reset (gv,[]);
                camset(gv,Camera);
                draw(bfeb,gv, struct ('x', geom, 'u', xscale*w,'colorfield',colorfield,'edgecolor',[1,1,1]*0.7, 'linewidth',2));
                %                 for  isovalue=linspace(min(wmag),max(wmag),20)
                %                     draw_isosurface(bfeb,gv, struct ('x', geom,'u',xscale*w,'scalarfield',wmagf,'isovalue',isovalue,'color',[0,0,0]));
                %                 end 
                draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omegas(i))/2/pi) ' [Hz]'],...
                    struct('fontsize', 18, 'fontname', 'Times'));
                draw_axes (gv,struct('length',R/3,'origin', [-R,+R,-R]));
                axis off
                pause(0.005);
            end
            pause(0.55);
            
        end
    end
    
    % Matrix of eigenvectors from equation (20) of [1]
    mE=W;
    % Check whether the eigenvectors are mass and stiffness normalized
    mId=zeros(size(mE,2))+inf;
    mOm=zeros(size(mE,2))+inf;
    for i=1:size(W,2)
        for k=1:size(W,2)
            mId(i,k)=mE(:,i)'*mM*mE(:,k);
            mOm(i,k)=mE(:,i)'*mK*mE(:,k);
        end
    end
    if (norm(mId -eye(size(mId )))>1e3*eps)
        warning('Eigenvectors not mass normalized');
    end
    if (norm(mOm -diag(Omegas))>1e3*eps*norm(Omegas))
        warning(['Eigenvectors not stiffness normalized: norm(mOm -Omega)=' num2str(norm(mOm -diag(Omegas)))]);
    end
    
    % Construct the acoustic-structural coupling matrix mG. This corresponds
    % to the matrix C*D, both defined below equation (16) of [1].
    % Material: note the mass density is set to 1.0 so that the "mass" 
    %     matrix below will correspond to the surface area. 
    dummy_prop = property_linel_iso (struct('E',1,'nu',0,'rho',1.0));
    dummy_mater = mater_defor_ss_linel_triax (struct('property',dummy_prop));
    % Finite element block.
    % Note that this function computes the outer unit normal at each 
    %     quadrature point given the tangent vectors along the coordinate curves.
    function n= surface_normal(x,tangent)
        n= cross(tangent(:,1),tangent(:,2));
        n=n/norm(n);
    end
    boundary_feb = feblock_defor_ss (struct ('mater',dummy_mater, 'gcells',boundary_gcells,...
        'integration_rule',tensprod_nq_rule(struct('dim' , 2, 'order', 1 )),'Rm', [],'N',@surface_normal));
    % D defined below equation (16) of [1].
    mD=0 * speye(count(boundary_gcells), count(boundary_gcells));
    for j=1:count(boundary_gcells)
        % Compute the area of each surface finite element
        mD(j,j)=measure(feblock(struct('gcells',subset(boundary_gcells,j),...
            'integration_rule',gauss_rule(2,2))),geom,@(x)1);
    end
    
    % mG=C*D
    G = start (sparse_sysmat, get(u, 'neqns'),count(boundary_gcells));
    G =assemble(G, acoustic_coupling_panels (boundary_feb, geom, u));
    mG =get(G,'mat');
    
    if debug
        P=P-1;
        vFrad=mG*(zeros(count(boundary_gcells),1)+1);
        
        Frad= scatter_sysvec (u,vFrad);
        gv=drawmesh({fens,boundary_gcells},'gcells','facecolor','y','facealpha',0.1);
        options.x=geom;
        options.u=0.005*Frad;
        options.color='r';
        show_field_as_arrows(gv,options);
    end
    
    % Construct the matrix of the coordinates for all the nodes on the surface
    xyz=get(geom,'values');
    eqnums=gather(P,1:count(fens),'eqnums');
    mX=zeros(length(find(eqnums~= 0)),3);
    for j=1:length(eqnums)
        if (eqnums(j) ~= 0)
            mX(eqnums(j),:)=xyz(j,:);
        end
    end
    % Construct the matrix of the connectivities of all the surface finite elements (panels)
    mConn=get(boundary_gcells, 'conn');
    for j=1:size(mConn,1)
        mConn(j,:)=eqnums(mConn(j,:));% need to renumber to match mX
    end
    
    if ( debug )
        gv=drawmesh({fens,boundary_gcells},'gcells','facecolor','y','facealpha',0.1);
        for j=1:length(eqnums)
            if (eqnums(j) ~= 0)
                draw_text(gv, mX(eqnums(j),:),[num2str(eqnums(j))], struct('fontsize', 10));
            end
        end
    end
    
    if ( debug )
        plot_output_mesh(mX,mConn);
    end
end

function plot_output_mesh(mX,mConn)
    fens=fenodeset(struct('id',(1:size(mX,1))','xyz',mX));
    boundary_gcells=gcellset_Q4(struct('conn', mConn));
    gv=drawmesh({fens,boundary_gcells},'gcells','facecolor','g','facealpha',0.1);
end