%% Buckling of shaft under torsion, clamped-clamped-sliding
% 

%% 
% Description:
% Shaft under torsion, circular cross-section
% Case 2 from Table 2.1 of J.H.  ARGYRIS,   P.C.  DUNNE  and  D.W. SCHARPF, ON LARGE  
% DISPLACEMENT-SMALL   STRAIN  ANALYSIS   OF  STRUCTURES 
% WITH  ROTATIONAL   DEGREES  OF  FREEDOM. COMPUTER  METHODS  
% IN APPLIED  MECHANICS  AND  ENGINEERING   14  (1978)  401-451 .
% Clamped at the bottom
% No rotation at the top, sliding  -- buckling factor 2.0

%% 
% This model uses the corotational nonlinear beam element.

% Parameters:
E=10000;
nu=0.3;
L= 42; % Length of the beam
R = 1/2; % radius
magn=pi*E*(pi*R^4/4)/L;% Applied torque

neigvs = 1;% one buckling parameter

%% 
% Mesh
n=8;
tolerance=L/n/100;
Parameters=struct ('A',pi*R^2,'I2',pi*R^4/4,'I3',pi*R^4/4,'IT',pi*R^4/2,...
        'x1x2_vector',[0 1 0], 'label',1);
[fens,gcells] = Beam_member([0,0,0;0,0,L],n,@gcellset_beam3,Parameters);

% The bottom end 
bottomn=fenode_select(fens, struct('box', update_box([],[0,0,0]), 'inflate', tolerance));
% The top end
topn=fenode_select(fens, struct('box', update_box([],[0,0,L]), 'inflate', tolerance));

prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
% Finite element block
feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));
% Initialize configuration variables
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
u=0*clone(geom,'transl');
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom,'nfens'),1)*reshape(eye(3),1,9)));
rot=0*clone(geom,'rotat');
% Define the displacement increment field
dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));

% Apply EBC's
% Bottom end is clamped
ebc_fenids=[bottomn];
ebc_prescribed=[1];
ebc_comp=[];
ebc_val=ebc_fenids*0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
% Top end is allowed rotation about the Z axis
ebc_fenids=(topn);
ebc_prescribed=1;
ebc_comp=[4,5];
ebc_val=ebc_fenids*0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
dchi  = apply_ebc (dchi);

% Number equations
dchi  = numbereqns (dchi);
neqns=get(dchi, 'neqns');

% Assemble the system matrix
K = start (sparse_sysmat, neqns);
K = assemble (K, stiffness(feb, geom, u, Rfield, dchi));
% Load
F = sysvec;
F = start (F, neqns);
evs = loads(nodal_load(struct ('id',topn,'dir',[6],'magn',[magn])), dchi);
F = assemble (F, evs);
% Solve
dchi = scatter_sysvec(dchi, K\F);

u=slice(dchi,(1:3),'u');
rot=slice(dchi,(4:6),'rot');

% Plot
gv=graphic_viewer;
gv=reset (gv,[]);
scale=20;
draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot, 'facecolor','red'));

xlabel('X');
ylabel('Y');
zlabel('Z');
hold on

draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
draw_axes(gv, struct([]));

%% 
% Compute the geometric stiffness matrix
scale_down=1/1000;
Kg = start (sparse_sysmat, neqns);
Kg = assemble (Kg, stiffness_geo(feb, geom, scale_down*u, update_rotation_field(Rfield,scale_down*rot), dchi));

%% 
% Solve the buckling problem
try
    % For small number of elements eigs tends to fail with
    %     ??? Error using ==> eigs at 257
    %     Error with ARPACK routine dsaupd: info = -9999 
    [W,Omega]=eigs(-(get(Kg,'mat')+get(Kg,'mat')')/2/scale_down,(get(K,'mat')+get(K,'mat')')/2,...
        neigvs,'lm');
    Omegas=(1./(diag(Omega)));
catch
    [W,Omega]=eig(-full(get(Kg,'mat')+get(Kg,'mat')')/2/scale_down,full(get(K,'mat')+get(K,'mat')')/2);
    [Omegas,ix]=sort(abs(diag(Omega)), 'descend');
    W=W(:,ix(1));
    Omegas=1./Omegas(1);
end


%
for i=1:neigvs
    disp(['  Eigenvector ' num2str(i) ' eigenvalue ' num2str(Omegas(i)) ]);
end
disp(' ')


%% 
% Present the buckling modes
gv=graphic_viewer;
gv=reset (gv,[]);
scale=5;
w=clone(dchi,'w'); % make a copy of u
for i=1:neigvs
    ur = scatter_sysvec(w, W(:,i));
    u=slice(ur,(1:3),'u');
    rot=slice(ur,(4:6),'rot');
    wv = get(u,'values');
    wmag = sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2);
    dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
    colors=map_data(dcm, wmag);
    colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
    gv=reset (gv,[]);
    draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
    draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot,'colorfield',colorfield));
    draw_axes(gv, struct([]));
    title(['  Eigenvector ' num2str(i) ' eigenvalue ' num2str(Omegas(i)) ]);
    pause(2); % next eigenvector
end


