%% Flagpole buckling under distributed axial load.

%%
% Material parameters (steel).
E=29000000;%29,000ksi
nu=0.3;
g=32.17*12; % in/second^2
rho=0.28/g;% Pound/g mass density per unit volume

%%
% Dimensions
L=24*12;% Length, in
%%
% Cross-sectional properties
Parameters=Beam_cross_section(struct('type','hollow_circle','Rint',3,'Rext',4));

%%
% Load parameters. Distributed axial load.
% Buckling parameter
q_buckle=7.84*(E*Parameters.I2/L^3);
q=[0;0;-q_buckle];% 0 lbf/in, 

%%
% Simulation control parameters.
graphics=~true;


%%
% The mesh.
n=2;%0.99568
n=3;%1.0049
n=4;%1.0038
n=5;%1.0025
n=6;%1.0016
n=12;%0.9998
% n=24;%0.99928 
Parameters.x1x2_vector=[1 0 0];
[fens,gcells] = Beam_member([[0 0 0];[0 0 L]],n,@gcellset_beam3,Parameters);
clampedn=fenode_select(fens, struct('box', update_box([],[0 0 0])));

%%
% Material properties
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
%%
% Finite element block
feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));


%%
% Construct the requisite fields, geometry and displacement
% Initialize configuration variables
geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
u0=0*geom0;
Rfield0 = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
% Define the displacement increment field
dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));

% Apply EBC's
ebc_fenids=clampedn;
ebc_prescribed=[1];
ebc_comp=1:6;% clamped
ebc_val=0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
dchi  = apply_ebc (dchi);

% Number equations
dchi  = numbereqns (dchi);
neqns=get(dchi, 'neqns');

%%
% Solver

if (graphics )
    gv=graphic_viewer;
end


%% 
% Forces
F = start (sysvec, neqns);
fi = force_intensity(struct('magn' ,q));
F = assemble (F, distrib_loads_global(feb, geom0, u0, Rfield0,dchi, fi));


%% 
% Stiffness matrix
K = start (sparse_sysmat, neqns);
K = assemble (K, stiffness(feb,geom0,u0,Rfield0,dchi));


%% 
% Linear analysis to determine internal forces.
dchi = scatter_sysvec(dchi, K\F); % Displacement increment

if (graphics )
    %%
    % Graphic display of the mesh and the deformed beam
    gv=reset (gv,[]);
    draw(feb, gv, struct ('x', geom0, 'u', u0, 'R', Rfield0, 'facecolor','none'));
    scale=L/15/norm(slice(dchi,(1:3),'transl'));
    wu1 = u0 + scale*slice(dchi,(1:3),'transl');   % increment displacement
    wRfield1=update_rotation_field(Rfield0,scale*slice(dchi,(4:6),'rotat'));
    draw(feb, gv, struct ('x', geom0, 'u', wu1, 'R', wRfield1, 'facecolor','yellow'));
    
    
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
    hold on
    
    draw_axes(gv, struct([]));
    camset (gv, [-191.7314 -234.2888  331.4447   32.8952   58.4503  118.4085         0         0    1.0000   30.5575]);
    pause(1)
end

%% 
% Update the geometry
u1=u0+slice(dchi,(1:3),'transl');
Rfield1=update_rotation_field(Rfield0,slice(dchi,(4:6),'rotat'));

%% 
% Compute geometric stiffness
Kg = start (sparse_sysmat, neqns);
Kg = assemble (Kg, stiffness_geo(feb,geom0,u1,Rfield1,dchi));


%%
% Buckling analysis
[W,Omega]=eig(-full((get(Kg,'mat')+get(Kg,'mat')')/2),full(get(K,'mat')+get(K,'mat')')/2);
[ignore,ix]= sort(diag(Omega)); 
Omegas=1/Omega(ix(end),ix(end));

disp([' Buckling parameter=' num2str(Omegas) ', compared with exact 1.000'])

if (graphics )
    %%
    % Graphic display of the mesh and the deformed beam
    gv=reset (gv,[]);
    draw(feb, gv, struct ('x', geom0, 'u', u0, 'R', Rfield0, 'facecolor','none'));
    dchi = scatter_sysvec(dchi, W(:,ix(end)));
    scale=L/5/norm(slice(dchi,(1:3),'transl'));
    wu1 = u1 + scale*slice(dchi,(1:3),'transl');   % increment displacement
    wRfield1=update_rotation_field(Rfield1,scale*slice(dchi,(4:6),'rotat'));
    draw(feb, gv, struct ('x', geom0, 'u', wu1, 'R', wRfield1, 'facecolor','yellow'));
    
    
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
    hold on
    
    draw_axes(gv, struct([]));
    camset (gv, [-191.7314 -234.2888  331.4447   32.8952   58.4503  118.4085         0         0    1.0000   30.5575]);
    pause(1)
end


% %%
% % Graphical presentation.
% figure
% plot([2,3,4,5,6,12,24],7.84/7.837*100*[0.99568,1.0049,1.0038,1.0025,1.0016,0.9998,0.99928 ],'rd-','linewidth',3)
% labels('Number of elements [ND]','Normalized buckling load [\%]')
% grid on
% grid  minor
% set_graphics_defaults
% snapnow; 

