%%  Simply-supported beam modal analysis. Corotational beam

%% 
% Parameters:
E=30002*1000;%30002ksi
nu=0.;
b=18; h=18; L=300;% in, cross-section and length
g=32.17*12; % in/second^2
rho=0.2/g;% Pound/g mass density per unit volume

%% 
% Cross-sectional properties
A=b*h;%in^2
I2=b*h^3/12;%cm^4 
I3=b^3*h/12;%cm^4 

%% 
% Choose the mass formulation:
consistent_no_rotation_inertia=0;
consistent_with_rotation_inertia=1;
lumped_diagonal_no_rotation_inertia=2;
lumped_diagonal_with_rotation_inertia=3;
mass_type=2;

%% 
% Analytical frequencies 
% 
% The analytical frequencies were taken from table 8-1 of Formulas for
% natural frequency and mode shape, Robert D. Blevins, Krieger publishing
% company, Malabar Florida, reprint edition 2001.
% 
% The beam has cylindrical supports at either end. Therefore in the
% vertical plane it behaves as a simply supported beam, while in the
% horizontal plane it behaves as a clamped beam.
% 
% Vertical plane, simply supported:
analyt_Frequencies(1)=(1*pi)^2/(2*pi*L^2)*sqrt(E*I2/rho/A);
% Horizontal plane, clamped:
analyt_Frequencies(2)=(4.73004074)^2/(2*pi*L^2)*sqrt(E*I2/rho/A);
neigvs=length(analyt_Frequencies);



%% 
% Select the number of elements per half the length.
n=4;
Parameters=struct ('A',A,'I2',I2,'I3',I3,'IT',I2+I3,...
    'x1x2_vector',[1 0 0], 'label',1);
[fens,gcells] = Beam_member([[0 -L/2 0];[0 L/2 0]],2*n,@gcellset_beam3,Parameters);
tolerance=L/n/100;

%% 
% Material properties
prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
mater = mater_defor_ss_linel_triax (struct('property',prop));
%% 
% Finite element block
feb = feblock_defor_ss_beam3_corot(struct ('mater',mater, 'gcells',gcells,'mass_type',mass_type));


%% 
% Construct the requisite fields, geometry and displacement

geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom,'nfens'),1)*reshape(eye(3),1,9)));

% Apply EBC's
ebc_fenids=fenode_select(fens, struct('box', update_box([],[0 -L/2 0]),'inflate', tolerance));
ebc_prescribed=[1];
ebc_comp=[1,2,3,5,6];% cylindrical
ebc_val=0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select(fens, struct('box', update_box([],[0 L/2 0]),'inflate', tolerance));
ebc_prescribed=[1];
ebc_comp=[1,2,3,5,6];% cylindrical
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ur  = apply_ebc (ur);

% Number equations
ur  = numbereqns (ur);
neqns=get(ur, 'neqns');

%% 
% Assemble the global discrete system

% Assemble the system stiffness matrix
K = start (sparse_sysmat, neqns);
K = assemble (K, stiffness(feb, geom, ur, Rfield));



 
M = start (sparse_sysmat, neqns);
M = assemble (M, mass(feb, geom, ur, Rfield));

options.tol =1e-26;
options.maxit = 5000;
options.disp = 0;
[W,Omega]=eigs(get(K,'mat'),get(M,'mat'), neigvs,'sm', options);
[Omegas2,ix]=sort((diag(Omega))) ;
Omegas2=somel(diag(Omega),ix);
Frequencies= sqrt(Omegas2)/2/pi;
  
%
for i=1:neigvs
    disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(Frequencies(i)) ' (analyt=' num2str(analyt_Frequencies(i)) ') [Hz]']);
end
disp(' ')

gv=graphic_viewer;
gv=reset (gv,[]);
scale=50;
w=clone(ur,'w'); % make a copy of u
for i=1:neigvs
    ur = scatter_sysvec(w, W(:,ix(i)));
    u=slice(ur,(1:3),'u');
    rot=slice(ur,(4:6),'rot');
    wv = get(u,'values');
    wmag = sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2);
    dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
    colors=map_data(dcm, wmag);
    colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
    for xscale=scale*sin((0:1:42)/21*2*pi)
        reset (gv,[]);
        title(['  Eigenvector ' num2str(i) ', frequency ' num2str(Frequencies(i)) ]);
                camset(gv,1.0e+003 *[ -1.3799   -1.7757    1.3005   -0.0102    0.0093    0.0015         0         0    0.0010    0.0029]);
        draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
        draw(feb, gv, struct ('x', geom, 'u', xscale*u, 'rot', xscale*rot,'colorfield',colorfield));
        draw_axes (gv,struct('length',L/10 ));
        pause(0.005);
    end
    pause(2); % next eigenvector
end
