%%  Argyris L-frame, out-of-plane load, nonlinear.

function CorotArgyrLFrameVibration
    %%
    % Parameters:
    E=71240.0;%MPa
    nu=0.31;% Poisson ratio
    rho=5e-9;
    b=3; h=30; L=240;% cross-sectional dimensions and length of each leg in millimeters
    Fmag= 5000;
    utol=1e-3;
    dt=0.001;
    tend=0.25;
    Newmarkg = 1/2; Newmarkb = 1/4*(1/2+Newmarkg)^2;
    % Newmarkg = 0.6; Newmarkb = 1/4*(1/2+Newmarkg)^2;
    maxit=24;
    graphics=0;
    Report =@(x) (disp(x) );
    
    %%
    % Mesh
    n=8;
    tolerance=L/n/100;
    Parameters=struct ('A',b*h,'I2',b*h^3/12,'I3',b^3*h/12,'IT',b^3*h/3,...
        'x1x2_vector',[0 0 1], 'label',1);
    [fens1,gcells1] = Beam_member([0,L,0;L,L,0],n,@gcellset_beam3,Parameters);
    [fens2,gcells2] = Beam_member([L,L,0;L,0,0],n,@gcellset_beam3,Parameters);
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2, gcells2, tolerance);
    gcells= cat(gcells1,gcells2);
    
    % The clamped end is
    clampedn=fenode_select(fens, struct('box', update_box([],[0,L,0]), 'inflate', tolerance));
    % The force is applied at the elbow
    elbown=fenode_select(fens, struct('box', update_box([],[L,L,0]), 'inflate', tolerance));
    % The force is applied at the elbow
    tipn=fenode_select(fens, struct('box', update_box([],[L,0,0]), 'inflate', tolerance));
    
    %%
    % Properties
    prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    
    %%
    % Finite element block
    feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));
    
    %%
    % Initialize configuration variables
    geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
    u=0*geom0;
    Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
    Rfield0=Rfield;
    % Define the displacement increment field
    dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));
    
    % Apply EBC's
    ebc_fenids=[clampedn];
    ebc_prescribed=[1];
    ebc_comp=[];
    ebc_val=0;
    dchi = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    dchi = apply_ebc (dchi);
    % Number equations
    dchi = numbereqns (dchi);
    neqns=get(dchi, 'neqns');
    
    
    if (graphics)
        gv=graphic_viewer;
    else
        plot_reference;
    end
    
    % Now comes the nonlinear solution
    tup = 1;
    dchi = dchi*0; % zero out the displacement
    v = dchi*0; % zero out the Velocity
    a = dchi*0; % zero out the Acceleration
    utol = 1e-15*get(dchi, 'neqns');
    
    K = start (sparse_sysmat, get(dchi, 'neqns'));
    M = start (sparse_sysmat, get(dchi, 'neqns'));
    
    
    t=0; %
    while (t <= tend)
        t = t + dt;
        Report(['Time ' num2str(t) ]); % pause
        % Initialization
        dchi = apply_ebc(dchi);% Apply boundary conditions
        u1 = u; % guess
        Rfield1=Rfield; % guess
        stepdchi = 0*apply_ebc(dchi);% Total increment in current step
        a1 = -(1/Newmarkb/dt)*v -(1/2-Newmarkb)/Newmarkb*a;
        v1 = v + dt*((1-Newmarkg)*a + Newmarkg*a1);
        dchipv =gather_sysvec(dt*v +(dt^2/2*(1-2*Newmarkb))*a);
        vpv =gather_sysvec(v +(dt*(1-Newmarkg))*a);
        nl = nodal_load(struct ('id',elbown,'dir',[3],'magn',[(t<0.2)*((t<0.1)*t+(t>0.1)*(0.2-t))*Fmag]));
        
        iter=1;
        while 1
            feb1=feb;
            F = start (sysvec, get(dchi, 'neqns'));
            F = assemble (F, loads(nl, dchi));
            [feb,evs] = restoring_force(feb,geom0,u1,Rfield1,dchi);       % Internal forces
            F = assemble (F, evs);
            K =assemble (K,cat(2,stiffness(feb,geom0,u1,Rfield1,dchi),stiffness_geo(feb,geom0,u1,Rfield1,dchi)));
            Km=get(K, 'mat');
            M =assemble (M,mass(feb,geom0,u1,Rfield1,dchi));
            Mm=get(M, 'mat');
            rhs=get(F,'vec')...
                +Mm*((-1/(Newmarkb*dt^2))*gather_sysvec(stepdchi)+(1/(Newmarkb*dt^2))*dchipv);
            dchi = scatter_sysvec(dchi, (Km+(1/(Newmarkb*dt^2))*Mm)\rhs); % Displacement increment
            %                 max(abs(get(F,'vec')))
            u1 = u1 + slice(dchi,(1:3),'transl');   % increment displacement
            stepdchi = stepdchi + dchi;
            v1 = v1 + (Newmarkg/Newmarkb/dt)*dchi;
            a1 = a1 + (1/Newmarkb/dt^2)*dchi;
            Rfield1=update_rotation_field(Rfield1,slice(dchi,(4:6),'rotat'));
            Report(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dchi))]);
            if (max(abs(gather(dchi,[],'values'))) < utol) break; end;% convergence check
            if (iter >maxit)% bailout for failed convergence
                error( [' Possible failed convergence']);
            end
            K = reinitialize (K);
            M = reinitialize (M);
            iter=iter+1;
        end
        [feb,ignore] = restoring_force(feb,geom0,u1,Rfield1,dchi);        % final update
        Report(['    Converged for t=' num2str(t)]); % pause
        u = u1;       % update the displacement
        Rfield=Rfield1; % update the rotations
        v = v1;       % update the velocities
        a = a1;       % update the accelerations
        
        if graphics
            gv=reset(gv,[]);
            draw(feb, gv, struct ('x', geom0, 'u', 0*u1, 'R', Rfield0, 'facecolor','r'));
            draw(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'facecolor','g'));
            %         draw_local_coordinates(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'length',12.5));
            %         camset(gv, 1.0e+003 *[-1.7885   -3.9427    1.7590
            %         0.0096    0.0029    0.0071         0         0    0.0010    0.0002]);
            drawnow;
        else
            tipu=gather(u1,tipn, 'values', 'noreshape');
            elbowu=gather(u1,elbown, 'values', 'noreshape');
            plot(t,tipu(1), 'rv');
            hold on
            plot(t,tipu(2), 'gv');
            plot(t,tipu(3), 'bv');
            plot(t,elbowu(1), 'ro');
            hold on
            plot(t,elbowu(2), 'go');
            plot(t,elbowu(3), 'bo');
            figure(gcf);
        end
    end
    function plot_reference()
        tipus =[-0.0000    0.0000    0.0105
            -0.0007    0.0001    0.0535
            -0.0040    0.0001    0.0815
            -0.0090   -0.0002   -0.0649
            -0.0087    0.0011   -0.4843
            0.0064    0.0115   -1.1190
            0.0474    0.0433   -1.8826
            0.1298    0.1117   -2.7206
            0.2837    0.2361   -3.6603
            0.5572    0.4490   -4.7895
            0.9912    0.7869   -6.0492
            1.6059    1.2690   -7.2278
            2.4193    1.8937   -8.1408
            3.4461    2.6507   -8.6994
            4.6746    3.5371   -8.8898
            6.0616    4.5473   -8.6847
            7.5437    5.6316   -7.9402
            9.0450    6.6889   -6.4574
            10.4715    7.6174   -4.1200
            11.7063    8.3629   -0.9478
            12.6324    8.9105    2.9600
            13.1385    9.2140    7.5903
            13.1235    9.1881   13.0286
            12.4988    8.7781   19.3188
            11.1699    7.9983   26.4061
            9.0865    6.9303   34.1354
            6.2517    5.6665   42.3523
            2.6460    4.2548   51.0410
            -1.7590    2.7480   60.2440
            -6.9355    1.2629   69.8807
            -12.7848   -0.0547   79.7617
            -19.1367   -1.0886   89.6865
            -25.8187   -1.7801   99.5467
            -32.7508   -2.1002  109.3328
            -39.8573   -2.0173  118.9469
            -46.9791   -1.5336  128.1940
            -53.9399   -0.7177  136.9593
            -60.5833    0.3282  145.2140
            -66.8548    1.5201  152.9207
            -72.8290    2.8023  160.0188
            -78.5666    4.1136  166.4769
            -84.0722    5.3739  172.3197
            -89.3369    6.5082  177.5572
            -94.3801    7.4661  182.1416
            -99.3070    8.2367  186.0798
            -104.2566    8.8342  189.4503
            -109.2585    9.2683  192.2769
            -114.2330    9.5434  194.5213
            -119.0508    9.6661  196.1746
            -123.5934    9.6593  197.3093
            -127.8130    9.5740  198.0035
            -131.6020    9.4514  198.2553
            -134.7428    9.2893  198.0753
            -137.0229    9.0613  197.5609
            -138.2093    8.7144  196.7770
            -138.1087    8.2008  195.7108
            -136.6996    7.5281  194.3689
            -134.0265    6.7272  192.7964
            -130.0890    5.8133  190.9901
            -124.8930    4.8035  188.8684
            -118.5461    3.7530  186.3457
            -111.2758    2.7529  183.3754
            -103.3845    1.8949  179.9135
            -95.1731    1.2280  175.9007
            -86.8509    0.7420  171.3386
            -78.5206    0.3886  166.2944
            -70.3243    0.1241  160.8347
            -62.5329   -0.0765  155.0123
            -55.3434   -0.2241  148.8587
            -48.8206   -0.3275  142.4556
            -42.9791   -0.4128  135.9784
            -37.7101   -0.4774  129.5735
            -32.9194   -0.4773  123.3300
            -28.7035   -0.3964  117.3922
            -25.1685   -0.2598  111.9558
            -22.2415   -0.0807  107.1840
            -19.7227    0.1758  103.1516
            -17.4580    0.5684   99.8667
            -15.4363    1.1161   97.3593
            -13.6868    1.7833   95.6929
            -12.1906    2.5167   94.8926
            -10.9225    3.2875   94.8820
            -9.8588    4.1107   95.5030
            -9.0264    4.9782   96.6520
            -8.5668    5.7874   98.3823
            -8.5349    6.4480  100.7361
            -8.8640    6.9673  103.6146
            -9.6493    7.3409  106.9208
            -11.0960    7.5107  110.6395
            -13.3633    7.3907  114.8278
            -16.5933    6.8988  119.5511
            -20.8382    6.0555  124.7494
            -26.0382    4.9769  130.2570
            -32.2242    3.7316  135.9736
            -39.5603    2.3144  141.8956
            -48.1138    0.7769  147.9931
            -57.7487   -0.7184  154.1090
            -68.2312   -1.9890  160.0485
            -79.3553   -2.9162  165.7379
            -90.9191   -3.4297  171.1888
            -102.6247   -3.4687  176.3340
            -114.1177   -3.0099  181.0039
            -125.1263   -2.0963  185.1256
            -135.5141   -0.8047  188.8195
            -145.1264    0.7827  192.2157
            -153.6420    2.5394  195.2867
            -160.8055    4.3177  198.0097
            -166.5851    6.0240  200.5142
            -170.9883    7.6032  202.8977
            -174.0479    9.0250  205.0988
            -175.8839   10.2846  207.0445
            -176.6172   11.3774  208.7850
            -176.3080   12.2811  210.3677
            -175.0260   12.9771  211.6756
            -172.9167   13.4724  212.5384
            -170.1183   13.7759  212.9104
            -166.7107   13.8742  212.8028
            -162.7215   13.7271  212.1310
            -158.0695   13.2651  210.7788
            -152.6546   12.4343  208.7596
            -146.5406   11.2510  206.1384
            -139.8182    9.7559  202.8278
            -132.4072    7.9723  198.6685
            -124.2476    5.9550  193.6481
            -115.4137    3.8057  187.8641
            -105.9486    1.6470  181.2893
            -95.9146   -0.3602  173.8462
            -85.5373   -2.0467  165.6833
            -75.0790   -3.3024  157.0831
            -64.6792   -4.0425  148.1937
            -54.4075   -4.1644  139.0320
            -44.4298   -3.6161  129.7666
            -35.0140   -2.4772  120.7553
            -26.3594   -0.8933  112.2738
            -18.5249    1.0264  104.3973
            -11.5279    3.1897   97.1220
            -5.4299    5.4442   90.5233
            -0.3136    7.5820   84.7449
            3.8143    9.4429   79.8427
            7.0839   10.9818   75.6802
            9.6289   12.1845   72.0401
            11.4465   12.9676   68.7875
            12.4404   13.2057   65.8888
            12.5483   12.8257   63.3263
            11.8023   11.8632   61.0346
            10.3277   10.4488   58.9330
            8.2946    8.7335   57.0594
            5.8361    6.8238   55.6579
            3.0597    4.8046   55.0553
            0.1501    2.8303   55.4239
            -2.6850    1.1038   56.7399
            -5.3458   -0.2464   58.9980
            -7.8451   -1.2125   62.3211
            -10.2255   -1.8290   66.8348
            -12.5155   -2.0818   72.4774
            -14.6840   -1.9120   78.9374
            -16.6959   -1.3299   85.8672
            -18.7085   -0.4770   93.1247
            -21.0331    0.4690  100.6656
            -23.9464    1.3849  108.3114
            -27.5632    2.1844  115.7676
            -31.8680    2.7718  122.8501
            -36.8460    3.0699  129.5619
            -42.5287    3.0916  135.9215
            -48.9105    2.9388  141.8407
            -55.8530    2.7332  147.2379
            -63.0970    2.5685  152.1725
            -70.2885    2.5236  156.7203
            -77.0946    2.6778  160.8549
            -83.2873    3.1038  164.5597
            -88.6672    3.8360  167.9154
            -92.9559    4.8342  170.9781
            -95.8580    5.9930  173.6937
            -97.2651    7.2248  176.0590
            -97.2756    8.5112  178.1923
            -96.0514    9.8531  180.1389
            -93.8064   11.2419  181.8021
            -90.8115   12.6533  183.0486
            -87.2785   14.0270  183.7568
            -83.4417   15.2963  183.8252
            -79.6617   16.4190  183.2205
            -76.1989   17.3303  181.9766
            -73.0601   17.9221  180.1601
            -70.1839   18.1217  177.8572
            -67.5165   17.9236  175.0801
            -64.8830   17.3656  171.7555
            -62.0518   16.5338  167.8178
            -58.8324   15.5415  163.1472
            -54.9952   14.4484  157.4733
            -50.2482   13.2574  150.5506
            -44.4060   12.0175  142.3996
            -37.5192   10.8180  133.1646
            -29.8789    9.7215  122.9095
            -22.0803    8.7572  111.7511
            -14.9844    7.8946   99.9452
            -9.4356    6.9684   87.6700
            -5.8531    5.7184   74.9734
            -4.0181    3.9665   61.8441
            -3.3513    1.8319   48.2147
            -3.3254   -0.2032   34.2251
            -3.4130   -1.5447   20.3390
            -2.9573   -1.8232    6.7738
            -1.7040   -1.0299   -6.9139
            -0.1748    0.5513  -21.1442
            0.7157    2.4271  -35.4756
            0.1900    3.9958  -49.1576
            -2.2211    4.9085  -62.1528
            -6.6069    5.3009  -74.8238
            -12.5329    5.6129  -87.3132
            -19.2990    6.3243  -99.8263
            -26.1183    7.7418 -112.3641
            -32.0908    9.7431 -124.1586
            -36.5353   11.9026 -134.3089
            -39.5903   14.1125 -142.8074
            -42.0822   16.6672 -150.3286
            -44.6057   19.6584 -157.2919
            -47.2786   22.7774 -163.7367
            -50.0472   25.6325 -169.6211
            -52.8418   27.9213 -174.7306
            -55.6725   29.6368 -179.0455
            -58.8006   31.2119 -183.0974
            -62.2502   32.9813 -186.9581
            -65.3523   34.6604 -189.8060
            -67.2734   35.6936 -191.0107
            -67.5741   35.8945 -190.5695
            -66.2079   35.4333 -188.6284
            -63.3480   34.6336 -185.5557
            -59.1563   33.6331 -181.8294
            -53.6131   32.1093 -177.2959
            -47.0752   29.8162 -171.4895
            -40.6656   27.0859 -164.5616
            -35.5109   24.3740 -157.0850
            -31.9926   21.7183 -149.1465
            -29.7410   18.8730 -140.3857
            -27.9192   15.6274 -130.4988
            -25.5207   11.9894 -119.2014
            -22.1122    8.4703 -106.8376
            -17.8389    5.7218  -94.1511
            -12.8386    3.9544  -80.9796
            -7.3868    2.9550  -66.3566
            -2.2984    2.5327  -50.2785
            1.3986    2.4810  -33.8744
            3.2116    2.3727  -17.8842
            3.2063    1.8625   -2.3048
            1.6707    1.0665   13.0461
            -1.0538    0.5586   28.1562
            -4.3961    0.9278   43.0185
            -7.4307    2.3610   57.6261
            -9.4100    4.5201   71.5808
            -10.3397    6.9545   84.5383];
        
        elbowus =[-0.0000   -0.0000    0.0884
            -0.0008   -0.0006    0.4002
            -0.0041   -0.0030    0.9927
            -0.0124   -0.0099    1.8757
            -0.0282   -0.0253    3.0646
            -0.0579   -0.0543    4.6594
            -0.1177   -0.1054    6.8397
            -0.2313   -0.1899    9.6987
            -0.4209   -0.3184   13.1636
            -0.7048   -0.4961   17.1308
            -1.1053   -0.7229   21.5616
            -1.6579   -1.0006   26.4832
            -2.4064   -1.3396   31.9190
            -3.3718   -1.7408   37.7516
            -4.5399   -2.1801   43.7750
            -5.8913   -2.6243   49.8459
            -7.4236   -3.0543   55.9032
            -9.1511   -3.4775   61.9448
            -11.0646   -3.8960   67.9216
            -13.1067   -4.2726   73.6954
            -15.2056   -4.5597   79.1462
            -17.3088   -4.7387   84.2230
            -19.4105   -4.8318   88.9546
            -21.5299   -4.8734   93.4004
            -23.6465   -4.8631   97.5502
            -25.6918   -4.7758  101.3400
            -27.6111   -4.6084  104.7496
            -29.4068   -4.3874  107.8308
            -31.1421   -4.1496  110.6929
            -32.8833   -3.9072  113.4355
            -34.6060   -3.6366  116.0519
            -36.2797   -3.3242  118.5391
            -37.9686   -2.9798  120.9896
            -39.7769   -2.5978  123.5158
            -41.7982   -2.1442  126.2181
            -44.0524   -1.5862  129.1307
            -46.5009   -0.9239  132.2135
            -49.1577   -0.1855  135.4499
            -52.0678    0.6173  138.8470
            -55.2299    1.4906  142.3799
            -58.6397    2.4147  146.0111
            -62.2552    3.3297  149.6485
            -65.9244    4.1606  153.1466
            -69.4960    4.8539  156.4141
            -72.8739    5.3767  159.4030
            -76.0031    5.7025  162.0712
            -78.8300    5.8167  164.3862
            -81.2084    5.7091  166.2940
            -83.0123    5.3771  167.7513
            -84.2723    4.8409  168.7722
            -85.0664    4.1511  169.4046
            -85.4526    3.3816  169.7195
            -85.5095    2.5987  169.7873
            -85.3133    1.8474  169.6421
            -84.9269    1.1704  169.3252
            -84.4180    0.6186  168.9174
            -83.8749    0.2368  168.5083
            -83.4173    0.0442  168.1769
            -83.1022    0.0228  167.9616
            -82.8756    0.1264  167.8518
            -82.7140    0.3053  167.8374
            -82.6676    0.5092  167.9095
            -82.7971    0.6859  168.0657
            -83.1098    0.7726  168.3065
            -83.4967    0.6782  168.5764
            -83.8112    0.3089  168.7924
            -83.9473   -0.3840  168.9134
            -83.8639   -1.3846  168.9409
            -83.6141   -2.6393  168.8861
            -83.2505   -4.0938  168.7285
            -82.7399   -5.6958  168.4140
            -82.0107   -7.3725  167.8927
            -81.0276   -9.0131  167.1435
            -79.8545  -10.5059  166.2088
            -78.6308  -11.7883  165.1880
            -77.3829  -12.8121  164.1228
            -76.0331  -13.5145  162.9949
            -74.6387  -13.8666  161.8541
            -73.3719  -13.8824  160.7966
            -72.4056  -13.6216  159.9262
            -71.8450  -13.1856  159.3623
            -71.6580  -12.6448  159.1719
            -71.8214  -12.0416  159.3861
            -72.4664  -11.4551  160.0687
            -73.7381  -10.9849  161.2600
            -75.6458  -10.6955  162.9156
            -78.0720  -10.5792  164.9300
            -80.8510  -10.5676  167.1898
            -83.8921  -10.6000  169.6127
            -87.1521  -10.6397  172.1115
            -90.4959  -10.6305  174.5582
            -93.7173  -10.4862  176.8207
            -96.6154  -10.1156  178.7782
            -99.0613   -9.4826  180.3564
            -101.0706   -8.6481  181.5761
            -102.6560   -7.6972  182.4778
            -103.7305   -6.6871  183.0780
            -104.2789   -5.6895  183.4261
            -104.4577   -4.8028  183.5903
            -104.5158   -4.1073  183.6314
            -104.6077   -3.6146  183.6121
            -104.7481   -3.2740  183.6019
            -104.9828   -3.0283  183.6781
            -105.4505   -2.8281  183.8893
            -106.2268   -2.6141  184.2452
            -107.2196   -2.3233  184.7471
            -108.3029   -1.9079  185.3669
            -109.4738   -1.3444  186.0319
            -110.7485   -0.6220  186.6757
            -111.9883    0.2493  187.2739
            -113.0237    1.2295  187.8020
            -113.7634    2.2476  188.1781
            -114.0974    3.2148  188.3005
            -113.8851    4.0699  188.1348
            -113.0529    4.7852  187.6922
            -111.6601    5.3376  186.9505
            -109.8462    5.7086  185.8907
            -107.6326    5.8799  184.5504
            -104.9725    5.8314  182.9754
            -102.0051    5.5529  181.2001
            -98.9565    5.0597  179.2890
            -95.9662    4.3997  177.3592
            -93.1246    3.6175  175.5230
            -90.5502    2.7249  173.8249
            -88.3580    1.7104  172.2909
            -86.5649    0.5683  170.9729
            -85.0871   -0.6986  169.9002
            -83.9000   -2.1085  169.0693
            -83.0277   -3.6922  168.4515
            -82.3161   -5.4298  167.9488
            -81.5205   -7.2201  167.4196
            -80.5310   -8.9470  166.7419
            -79.3415  -10.5186  165.8222
            -77.8941  -11.8259  164.5835
            -76.0177  -12.7193  162.9476
            -73.5497  -13.0525  160.8414
            -70.5201  -12.7781  158.2533
            -67.1538  -11.9773  155.2562
            -63.6816  -10.7946  151.9507
            -60.2129   -9.3873  148.4133
            -56.8193   -7.9406  144.7449
            -53.6375   -6.6825  141.1323
            -50.8611   -5.8429  137.8303
            -48.6227   -5.5751  135.0589
            -46.9119   -5.9141  132.9165
            -45.6262   -6.7802  131.3676
            -44.7036   -8.0336  130.3316
            -44.1986   -9.5276  129.7826
            -44.1468  -11.0841  129.6768
            -44.4418  -12.4680  129.8728
            -44.8589  -13.4257  130.1827
            -45.1811  -13.7761  130.4712
            -45.3634  -13.4809  130.7296
            -45.4911  -12.5989  131.0096
            -45.6284  -11.2032  131.3210
            -45.7601   -9.3684  131.6056
            -45.8746   -7.2260  131.8156
            -46.0501   -4.9773  131.9997
            -46.4133   -2.8226  132.2903
            -47.0067   -0.8941  132.7791
            -47.6891    0.7297  133.4025
            -48.3089    1.9571  134.0672
            -48.8511    2.7059  134.7488
            -49.3698    2.9763  135.4448
            -49.9088    2.8540  136.1538
            -50.4430    2.4410  136.8420
            -50.9177    1.8103  137.4483
            -51.2927    1.0393  137.9211
            -51.5524    0.2544  138.2529
            -51.7463   -0.3903  138.5016
            -51.9647   -0.7675  138.7270
            -52.2308   -0.8069  138.9193
            -52.4199   -0.4951  138.9960
            -52.4179    0.1418  138.9055
            -52.2034    1.0513  138.6323
            -51.7691    2.1720  138.1486
            -51.0825    3.4452  137.4108
            -50.0829    4.7947  136.3195
            -48.7683    6.1224  134.7669
            -47.1747    7.3295  132.6894
            -45.3003    8.3271  130.0744
            -43.1012    9.0098  126.9163
            -40.5183    9.2427  123.1756
            -37.5011    8.9248  118.8180
            -34.0618    8.0572  113.8528
            -30.3698    6.7529  108.3570
            -26.7139    5.2236  102.5008
            -23.3237    3.7525   96.4908
            -20.2840    2.6106   90.4607
            -17.6004    1.9610   84.4718
            -15.2367    1.8199   78.5534
            -13.0804    2.0764   72.6427
            -10.9967    2.5536   66.5855
            -8.9785    3.0799   60.3135
            -7.1492    3.5167   53.9444
            -5.6273    3.7243   47.7393
            -4.3746    3.4811   41.7453
            -3.2358    2.5720   35.5885
            -2.1127    1.0736   28.8427
            -1.0609   -0.5420   21.3463
            -0.2814   -1.6434   13.4239
            0.0901   -1.8256    5.5227
            0.0686   -1.0741   -2.2866
            -0.2827    0.2973  -10.0877
            -0.8888    1.7551  -17.7039
            -1.6553    2.7508  -24.8227
            -2.5236    2.9817  -31.4463
            -3.5292    2.5354  -37.8179
            -4.7483    1.7773  -44.1152
            -6.1659    1.0835  -50.3250
            -7.6632    0.6599  -56.2798
            -9.0955    0.5568  -61.6281
            -10.3821    0.8090  -66.0803
            -11.6278    1.5316  -69.8794
            -13.0255    2.8621  -73.5240
            -14.6644    4.7193  -77.1999
            -16.3992    6.6740  -80.6116
            -17.7875    8.1297  -83.1482
            -18.4126    8.6860  -84.4026
            -18.3356    8.3457  -84.5780
            -17.9657    7.4489  -84.2561
            -17.5853    6.4169  -83.8315
            -17.1692    5.4556  -83.2022
            -16.6463    4.6457  -82.1125
            -16.1027    4.1825  -80.6462
            -15.6488    4.3024  -79.1191
            -15.3343    5.0521  -77.7960
            -15.0435    6.0971  -76.5792
            -14.4459    6.8579  -74.9194
            -13.3191    6.9704  -72.3563
            -11.8694    6.5306  -69.0354
            -10.5229    5.8871  -65.5708
            -9.4887    5.2258  -62.3783
            -8.6063    4.3870  -59.1817
            -7.5638    3.1491  -55.2937
            -6.2723    1.6005  -50.4811
            -4.9291    0.1793  -45.2013
            -3.7360   -0.6220  -39.9889
            -2.7678   -0.5857  -34.9650
            -1.9922    0.1208  -29.7347
            -1.3922    1.1178  -24.1257
            -0.9758    1.9729  -18.7183
            -0.6862    2.2500  -13.8289
            -0.4189    1.6845   -8.9286
            -0.1261    0.4779   -3.3289
            0.0276   -0.7087    3.2139
            -0.1866   -1.2125   10.4786
            -0.7618   -0.8736   17.9079
            -1.5248    0.0026   24.9104
            -2.3745    1.0528   31.3244];
        
        
        t=linspace(0, 0.25, size(tipus,1))
        plot(t,tipus(:,1), 'r-');
        hold on
        plot(t,tipus(:,2), 'g-');
        plot(t,tipus(:,3), 'b-');
        plot(t,elbowus(:,1), 'r-');
        hold on
        plot(t,elbowus(:,2), 'g-');
        plot(t,elbowus(:,3), 'b-');
        figure(gcf);
    end
end
