%% Free vibration of simply-supported beam with axial load.

%%
% Material parameters (steel).
E=29000000;%29,000ksi
nu=0.3;
g=32.17*12; % in/second^2
rho=0.28/g;% Pound/g mass density per unit volume

%%
% Dimensions
b=0.25;%in
h=4;%in
L=24;% in
%%
% Cross-sectional properties
Parameters=Beam_cross_section(struct('type','rectangle','d2',h,'d3',b));

%%
% Load parameters.
% transverse load
q=[0;0;-1];% 0 lbf/in, 
q=[0;0;0];% 0 lbf/in, 
% Buckling axial force
P_Euler=(pi^2*E*Parameters.I2/L^2);

%%
% Exact deformation under the transverse load, without the contribution of the axial load.
deflex=5*norm(q)*L^4/(384*E*Parameters.I2);

%%
% Simulation control parameters.
utol=1e-2;% convergence tolerance
graphics=~true;
neigvs=10;

%%
% The mesh.
n=6;
Parameters.x1x2_vector=[1 0 0];
[fens,gcells] = Beam_member([[0 -L/2 0];[0 L/2 0]],2*n,@gcellset_beam3,Parameters);
midn=fenode_select(fens, struct('box', update_box([],[0 0 0])));
forcen=fenode_select(fens, struct('box', update_box([],[0 -L/2 0])));

%%
% Material properties
prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
mater = mater_defor_ss_linel_triax (struct('property',prop));
%%
% Finite element block
feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));


%%
% Construct the requisite fields, geometry and displacement
% Initialize configuration variables
geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
u=0*geom0;
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
Rfield0=Rfield;
% Define the displacement increment field
dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));

% Apply EBC's
ebc_fenids=forcen;
ebc_prescribed=[1];
ebc_comp=[1,3,5];% cylindrical, with compressive force loading
ebc_val=ebc_fenids*0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select(fens, struct('box', update_box([],[0 L/2 0])));
ebc_prescribed=[1];
ebc_comp=[1,2,3,5];% cylindrical
ebc_val=ebc_fenids*0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
dchi  = apply_ebc (dchi);

% Number equations
dchi  = numbereqns (dchi);
neqns=get(dchi, 'neqns');

%%
% Solver

if (graphics )
    gv=graphic_viewer;
end

load_parameters=[0,linspace(1.0,1.2,11),linspace(0.8,1.0,11),linspace(-1,0.8,11),];
lowest_frequency=[];
for load_parameter=load_parameters
    
    % Initialization
    dchi = 0*dchi; % guess
    dchi = apply_ebc(dchi);
    u1 = u;
    Rfield1=Rfield;
    nl=nodal_load(struct ('id',forcen,'dir',[2],'magn',[load_parameter*P_Euler]));
    fi = force_intensity(struct('magn' ,q ));
    
    iter=1;
    while 1
        feb1=feb;  % Make a copy of the state
        F = start (sysvec, neqns);
        F = assemble (F, distrib_loads_global(feb, geom0, u1,Rfield1,dchi, fi));
        F = assemble (F, loads(nl, dchi));
        [feb1,evs] = restoring_force(feb1,geom0,u1,Rfield1,dchi);       % Internal forces
        F = assemble (F, evs);
        K = start (sparse_sysmat, neqns);
        K = assemble (K, cat(2,stiffness(feb1,geom0,u1,Rfield1,dchi),stiffness_geo(feb1,geom0,u1,Rfield1,dchi)));
        dchi = scatter_sysvec(dchi, K\F); % Displacement increment
        % Line search
        R0 = dot(get(F,'vec'),gather_sysvec(dchi));
        F = start (sysvec, neqns);
        F = assemble (F, loads(nl, dchi));
        [feb1,evs] = restoring_force(feb1,geom0,...
            u1+slice(dchi,(1:3),'transl'),update_rotation_field(Rfield1,slice(dchi,(4:6),'rotat')),dchi);       % Internal forces
        F = assemble (F, evs);
        R1 = dot(get(F,'vec'),gather_sysvec(dchi));
        a = R0/R1;
        if ( a<0 )
            eta = a/2 +sqrt((a/2)^2 -a);
        else
            eta =a/2;
        end
        eta=min( [eta, 1.0] );% line search concluded
        u1 = u1 + eta*slice(dchi,(1:3),'transl');   % increment displacement
        Rfield1=update_rotation_field(Rfield1,eta*slice(dchi,(4:6),'rotat'));
                disp(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dchi))]);
        if (max(abs(gather(dchi,[],'values'))) < utol) break; end;                    % convergence check
        iter=iter+1;
    end
    [feb,ignore] = restoring_force(feb,geom0,u1,Rfield1,dchi);        % final update
    disp(['    Converged for load_parameter=' num2str(load_parameter)]); % pause
    u = u1; % update the displacement
    Rfield=Rfield1; % update the rotations
    
    if (graphics )
        %%
        % Graphic display of the mesh and the deformed beam
        gv=reset (gv,[]);
        draw(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'facecolor','none'));
        pause(0.1)
        
        xlabel('X');
        ylabel('Y');
        zlabel('Z');
        hold on
        
        draw_axes(gv, struct([]));
        camset (gv, [ -244.7965 -277.9378  215.5132  -20.1699   14.8013    2.4770         0         0    1.0000   19.1924]);
        pause(1)
    end
    
    
%% 
% Free-vibration with the effect of internal stresses.
    K = start (sparse_sysmat, neqns);
    K = assemble (K, cat(2,stiffness(feb,geom0,u1,Rfield1,dchi),stiffness_geo(feb,geom0,u1,Rfield1,dchi)));
    M = start (sparse_sysmat, neqns);
    M = assemble (M, mass(feb,geom0,u1,Rfield1,dchi));
    Km=get(K, 'mat');
    Mm=get(M, 'mat');
    
    %     [W,Omega]=eigs(Km,Mm, neigvs,'sm');
    %     [Omegas2,ix]=sort((diag(Omega))) ;
    %     Frequencies= sqrt(Omegas2)/2/pi;
    %     for i=1:neigvs
    %         disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(Frequencies(i)) '  [Hz]']);
    %     end 
    [W,Omega]=eig(full(Km),full(Mm));
    [Omegas2,ix]=sort((diag(Omega))) ;
    for j=1:3
         Frequencies(j)= sign(Omegas2(j))*sqrt(abs(Omegas2(j)))/2/pi;
    end 
   
    W=W(:,ix);
    lowest_frequency=[lowest_frequency;Frequencies];
    disp(['  Lowest natural frequency '  num2str(Frequencies) '  [Hz]']);

    if (graphics )
        %%
        % Graphic display of the mesh and the deformed beam
        gv=reset (gv,[]);
        draw(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'facecolor','none'));
        dchi = scatter_sysvec(dchi, W(:,1));
        scale=L/10/norm(slice(dchi,(1:3),'transl'));
        wu1 = u1 + scale*slice(dchi,(1:3),'transl');   % increment displacement
        wRfield1=update_rotation_field(Rfield1,scale*slice(dchi,(4:6),'rotat'));
        title(['  Frequency ' num2str(Frequencies(1)) ]);
        draw(feb, gv, struct ('x', geom0, 'u', wu1, 'R', wRfield1, 'facecolor','yellow'));
        
    
        xlabel('X');
        ylabel('Y');
        zlabel('Z');
        hold on
        
        draw_axes(gv, struct([]));
        camset (gv, [ -244.7965 -277.9378  215.5132  -20.1699   14.8013    2.4770         0         0    1.0000   19.1924]);
        pause(1)
    end
end


%% 
% Present the lowest natural frequency as a function of the axial load.
analyt_frequency_1=(1*pi)^2/(2*pi*L^2)*sqrt(E*Parameters.I2/rho/Parameters.A);
figure
    plot(load_parameters,lowest_frequency(:,1)/analyt_frequency_1,'ro','linewidth',3)
    hold on
    plot(load_parameters,lowest_frequency(:,2)/analyt_frequency_1,'gx','linewidth',3, 'markersize',8)
    plot(load_parameters,lowest_frequency(:,3)/analyt_frequency_1,'bd','linewidth',3)
    labels('$P/P_{Euler}$ [ND]','Normalized frequency($P/P_{Euler}$) [ND]')
    grid on
    set_graphics_defaults