%% Spinning drill string about an offset axis. Steady-state analysis in rotating coordinate system.
%
%
%%
% This function implements the direct numerical simulation of the spinning
% beam (drill string) offset with respect to the axis of rotation.
function spinning_modal
    % Parameters:  The Dykstra thesis specifies the material and cross-sectional property data
    % in the second paragraph on page 90.
    %
    E=30002*1000;%Young's modulus 30002ksi
    nu=0.3;% Poisson ratio
    radius_internal=2.5/2;%in
    radius_external=6.25/2;%in
    L=45*12;% in
    g=32.17*12; % in/second^2
    rho=490/12^3/g; %lb/ft^3
    eccentricity=0.1;% axis eccentricity in inches
    zeta= 0.05; % damping ratio as selected in the thesis
    zeta_1= zeta; % damping ratio for mode 1
    zeta_2= zeta; % damping ratio for mode 2
    omega_1=2*pi*1.832; % computed previously for this structure in stival_4_7
    omega_2=2*pi*7.328; % computed previously for this structure in stival_4_7
    % solve for the Rayleigh damping parameters from a given damping ratio
    % for the first two modes of transverse vibration
    Rayleigh_damping_mass= 1/(omega_2^2-omega_1^2)*[omega_2^2,-omega_1^2]*2*[zeta_1*omega_1;zeta_2*omega_2];
    Rayleigh_damping_stiffness= 1/(omega_2^2-omega_1^2)*[-1,1]*2*[zeta_1*omega_1;zeta_2*omega_2];
    
    
    %%
    % Choose the mass formulation:
    consistent_no_rotation_inertia=0;
    consistent_with_rotation_inertia=1;
    lumped_diagonal_no_rotation_inertia=2;
    lumped_diagonal_with_rotation_inertia=3;
    mass_type=1;
    
    neigvs=2;
    
    %%
    % Output control
    Report =@(x) (disp(x) );
    Report =@(x) (0);
    graphics= false;
    live_plotting= false;
    
    %%
    % The geometry.
    X=[[-L/2 eccentricity 0];[L/2 eccentricity 0]];
    
    %%
    % Cross-sectional properties
    A=pi*(radius_external^2-radius_internal^2);%in^2
    I2=pi/4*(radius_external^4-radius_internal^4);;%cm^4
    I3=pi/4*(radius_external^4-radius_internal^4);;%cm^4
    IT=pi/2*(radius_external^4-radius_internal^4);
    
    %%
    % Construct the finite element mesh.
    n=3;
    Parameters=struct ('A',A,'I2',I2,'I3',I3,'IT',IT,...
        'x1x2_vector',[0 1 0], 'label',1);
    [fens,gcells] = Beam_member(X,2*n,@gcellset_beam3,Parameters);
    midn=fenode_select(fens, struct('box', update_box([],mean(X))));
    end1n=fenode_select(fens, struct('box', update_box([],X(1,:))));
    end2n=fenode_select(fens, struct('box', update_box([],X(2,:))));
    
    %%
    % Material properties
    prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    %%
    % Finite element block
    feb = feblock_defor_ss_beam3_corot(struct ('mater',mater, 'gcells',gcells,...
        'mass_type', mass_type,...
        'Rayleigh_damping_stiffness',Rayleigh_damping_stiffness,...
        'Rayleigh_damping_mass',Rayleigh_damping_mass));
    
    
    %%
    % Construct the requisite fields, geometry and displacement
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    % Define the displacement field
    ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));
    Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom,'nfens'),1)*reshape(eye(3),1,9)));
    
    
    %%
    %  Apply EBC's: the two pins are restrained against translation, and the
    %  rotation about the X axes is prescribed (the spin!).
    
    ur  = set_ebc(ur, end1n, 1, [1,2,3,4], 0);
    ur  = set_ebc(ur, end2n, 1, [1,2,3,4], 0);
    ur  = apply_ebc (ur);
    
    % Number equations
    ur  = numbereqns (ur);
    neqns=get(ur, 'neqns');
    
    
    %%
    % The elastic stiffness.
    K = start (sparse_sysmat, neqns);
    K = assemble (K, stiffness(feb, geom, ur, Rfield));
    Km =get(K,'mat');
    
    %  The mass matrix.
    M = start (sparse_sysmat, neqns);
    M = assemble (M, mass(feb, geom, ur, Rfield));
    Mm =get(M,'mat');
    
    C = start (sparse_sysmat, neqns);
    C =assemble (C,damping_proportional(feb, geom, ur,Rfield));
    Cm=get(C, 'mat');
    
    %%
    % Execute the simulation for these rotary speeds.
    rotary_speeds=200:-2:0;
    %     rotary_speeds=25:25:50;
    umnum=[];
    umana=[];
    for rotary_speed=rotary_speeds;%rotary speed, RPM
        
        ur=0*ur;
        %%
        % Spin parameters.
        freq=rotary_speed/60;% circular frequency in hertz
        Spin_x=freq*2*pi;% Angular frequency of spin
        
        
        if graphics, gv=graphic_viewer; end
        
        %         %%
        %         %   The parameters for the eigenvalue problem are set as:
        %         options.tol =1e-26;
        %         options.maxit = 5000;
        %         options.disp = 0;
        %
        %         %%
        %         % For a zero pre-stressing force we get the unmodified estimate of the first frequency:
        %         [W,Omega]=eigs(Km,Mm, neigvs,'sm', options);
        %         [Omegas2,ix]=sort((diag(Omega))) ;
        %         Frequencies= sqrt(Omegas2)/2/pi
        %         for i=1:neigvs
        %             disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(Frequencies(i)) '  [Hz]']);
        %         end
        %         omega_1=2*pi*Frequencies(1);
        %         Psi_1=W(:,ix(1));
        
        %  The gyroscopic (Coriolis) matrix is computed next.
        G = start (sparse_sysmat, neqns);
        G = assemble (G, coriolis_x_only(feb, geom, ur, Rfield, Spin_x));
        Gm =get(G,'mat');
        
        %         %                 A= [zeros(size(Km)),eye(size(Km));-Mm\Km,zeros(size(Km))] ; %-0*Mm\Cm];
        %         A= [zeros(size(Km)),eye(size(Km));-Mm\Km,-Mm\(Gm+Cm)]; %];
        %         [V,D]=  eig(full(A));
        %         dD=diag(D );
        %         ix=find(imag(dD)~=0);
        %         Wd=V(1:get(ur, 'neqns'),ix);
        %         omegad=dD(ix);
        %         Omegad=diag(omegad);
        
        F = start (sysvec, get(ur, 'neqns'));
        F = assemble (F, centrifugal_loads(feb, geom, ur,Rfield, [Spin_x,0,0], [0,0,0]));
        
        Lm =get(F, 'vec');
        
        U=((Km+1i*Spin_x*(Gm+Cm)-Spin_x^2*Mm))\(Lm);
        emid=gather(ur,midn,'eqnums');
        umid=U(emid(emid~=0));
        umnum=[umnum,abs(umid(2)+umid(3))]
        
    end
    
    %%
    % Graphical presentation of the results.
    
    %%
    % The analytical solution is in the form of (4.58).
    function v=uampl_analyt(Omega,omega_1,eccentricity)
        e=4*eccentricity/pi;
        beta=Omega./omega_1;
        v=e*beta.^2./sqrt((1-beta.^2).^2+(2*zeta*beta).^2);
    end
    
    rss=linspace(0, 200, 200);
    figure;
    plot(rss,uampl_analyt(rss/60*2*pi,omega_1,eccentricity),'r-','linewidth',3)
    hold on
    
    %%
    % The numerical solution matches the analytical predictions fairly well.
    % Note that the numerical model is fully nonlinear and  the
    % resonant frequency is therefore shifted a little bit higher than predicted by the
    % linear model.
    plot(rotary_speeds,umnum,'x','linewidth',3)
    labels('Rotary speed [rpm]','Maximum displacement [in]')
    legend({' Formula (4.58)',' Numerical'}, 'location', 'best')
    grid on
    set_graphics_defaults
end