% Meshing 
%
% Selection functions:
%   fenode_select           - Select nodes.
%   gcell_select            - Select geometric cells.
%   connected_nodes         - Extract the node numbers of the nodes  connected by given gcells. 
%   v_select                - Select locations (vertices).
%   find_unconn_fens        - Find nodes that are not connected to any cell in the array gcells.
%   bounding_box            - Compute the bounding box of a collection of points.
%   boxes_overlap           - Do the given boxes overlap?
%   in_box                  - Is the given location inside a box?
%   inflate_box             - Inflate the box.
%   update_box              - Update a box with another location.
%
% Mesh manipulation functions:
%   mesh_bdry               - Extract the boundary gcells from a mesh.
%   merge_meshes            - Merge two meshes together by gluing together nodes within tolerance.
%   merge_nodes             - Merge by gluing together nodes located within tolerance of each other.
%   mirror_mesh             - Mirror a 2-D mesh in a plane given by its normal and one point. 
%   rotate_mesh             - Rotate a mesh around a vector anchored at the given point. 
%   onto_sphere             - Project nodes onto a sphere of given radius.
%   transform_2_helix       - Generate the mesh of an helix. 
%   transform_apply         - Apply geometry transformation to the locations of the nodes.
%   block_function_shift    - Shift nodes in a three-dimensional block using a displacement function.
%   renumber_gcell_conn     - Renumber the nodes in the connectivity of the geometric cells based on a new
%   compact_fens            - Compact the finite element node array by deleting unconnected nodes.
%   validate_mesh           - Validate finite element mesh.
%   renum_mesh              - Renumber the nodes and the cells using an optimal numbering of the finite
%   renum_mesh_numbering    - Compute an optimal numbering of the finite element nodes 
%   renum_mesh_renumber     - Renumber the nodes and the geometric cells from a given numbering
%   conn_to_sparse          - Construct a sparse matrix from connectivities.
%   gcells_to_conns         - Make  a cell array of connectivities for use in mesh renumbering routines.
%   smoothing               - General smoothing of meshes.
% 
% Curve mesh generation utilities:
%   L2_block                - Mesh of a line segment located in a given range, L2 Elements
%   L2_blockx               - Mesh of a 1-D block, L2 cells. The nodes are located at the supplied
%   L2_extrude              - Extrude a mesh of line segments into a mesh of quadrilaterals.
%   L2_to_L4                - Convert a mesh of L2 elements (two-node) to L4 line elements.
% 
% Hexahedral mesh functions:
%   H8_block                - Mesh of a 3-D block located in a given range, H8 cells
%   H8_block_shift          - Shift nodes in a three-dimensional block using a displacement function.
%   H8_blockx               - Mesh of a 3-D block, H8 cells. The nodes are located at the Cartesian
%   H8_cylinder             - Make H8 mesh of a solid  cylinder
%   H8_hexahedron           - Mesh of a general hexahedron given by the location of the vertices
%   H8_hollow_cylinder      - Make H8 mesh of a solid hollow cylinder
%   H8_refine               - Refine a mesh of hexahedrals by octasection.
%   H8_sphere               - Create a solid mesh of 1/8 of the sphere of "radius", using octasection. 
%   H8_sphere_n             - Create a solid mesh of 1/8 of the sphere of "radius", with given number of elements. 
%   H8_to_H27               - Convert a mesh of hexahedron H8 to hexahedron H27.
%   H8_to_H64               - Convert a mesh of hexahedron H8 to hexahedron H64.
%   H8_to_T4_random         - Convert a mesh of quadrilateral Q4's to 2 triangles T3 each.
%   H20_block               - Mesh of a block located in a given range, H20 cells
%   H27_block               - Mesh of a orthogonal block in a given range, H27 cells.
% 
% Quadrilateral mesh generation:
%   Q16_block               - Block mesh of Q16 elements 
%   Q4_annulus              - Mesh of an annulus segment.
%   Q4_block                - Mesh of a rectangle located in a given range, Q4 elements
%   Q4_blocku               - Mesh of a rectangle located in a given range (Unstructured mesh).
%   Q4_blockx               - Mesh of a 2-D block, Q4 cells. The nodes are located at the Cartesian
%   Q4_circle               - Create a mesh of a quarter circle.
%   Q4_elliphole            - Mesh of one quarter of a rectangular plate with an elliptical hole
%   Q4_extrude              - Extrude a mesh of quadrilaterals into a mesh of hexahedra.
%   Q4_quadrilateral        - Mesh of a general quadrilateral given by the location of the vertices.
%   Q4_refine               - Refine a mesh of quadrilaterals by bisection
%   Q4_sphere               - Create a surface mesh of 1/8 of the sphere of "radius". 
%   Q4_sphere_n             - Create a surface mesh of 1/8 of the sphere of "radius", with given number of elements. . 
%   Q4_to_Q16               - Convert a mesh of Quadrilateral Q4 to Quadrilateral Q16.
%   Q4_to_Q9                - Convert a mesh of quadrilateral Q4 to quadrilateral Q9.
%   Q8_block                - Block mesh of Q8 elements 
%   Q9_block                - Block mesh of Q9 elements 
% 
% Triangle mesh generation:
%   T3_ablock               - Mesh of T3 elements of a rectangle located in a given range
%   T3_ablockc              - Mesh of T3 elements of a rectangle located in a given range
%   T3_ablockn              - Mesh of T3 elements of a rectangle located in a given range
%   T3_annulus              - Mesh of an annulus segment , triangles T3.
%   T3_block                - Mesh of a rectangle located in a given range
%   T3_blockc               - Mesh of a rectangle located in a given range. Corners are bisected.
%   T3_blockn               - Mesh of a rectangle located in a given range.  Never bisects the corners.
%   T3_blocku               - Mesh of a rectangle located in a given range.  An unstructured mesh.
%   T3_cblock               - Mesh of a rectangle located in a given range
%   T3_cblock2d             - Mesh of a rectangle located in a given range
%   T3_crossblock           - Mesh of a rectangle located in the range (Cross-diagonal).
%   T3_mesh_sizes           - Estimate T3 mesh size from element wise errors and convergence rate
%   T3_to_1Q4               - Convert a mesh of triangles T3 to one quadrilateral Q4 each.
%   T6_block                - Block mesh of T6 elements 
%   targe2_mesher           - Automatic triangulation of a general 2-D domain. 
%                        The input is given as a cell array Of commands.
%   targe2_mesher_vl        - Create triangulation of a domain given as an array of vertices. No holes
%                        polygon.
%   targe2_mesher_adapt     - Automatic graded triangulation of a general 2-D domain. 
% 
% Tetrahedral mesh generation:
%   T4_blob                 - Tetrahedral (t4) Mesh of a block located in a given range
%   T4_block                - Tetrahedral (t4) Mesh of a block located in a given range.
%   T4_block_wslivers       - Tetrahedral (t4) Mesh of a block located in a given range
%   T4_blocka               - Tetrahedral (t4) Mesh of a block located in a given range. This function
%   T4_blockb               - Tetrahedral (t4) Mesh of a block located in a given range. This function
%   T4_blockc               - Tetrahedral (t4) Mesh of a block located in a given range.  The
%   T4_blockc2              - Tetrahedral (t4) Mesh of a block located in a given range.  The
%   T4_blockdel             - Tetrahedral (t4) Delaunay Mesh of a block located in a given range
%   T4_blockrandom          - Tetrahedral (t4) Mesh of a block located in a given range. This function
%   T4_cylinderdel          - Tetrahedral (T4) Delaunay Mesh of a cylinder of Length and Radius.
%   T4_refine               - Refine a mesh of 4-node tetrahedra by octasection
%   T4_volume               - function Volumes =t4_volume (t,v)
%   T10_block               - Tetrahedral (T10) Mesh of a block located in the range
%   Abaqus_mesh_import      - Import 10-node tetrahedra ABAQUS Mesh.
% 
% Tetrahedral mesh utilities:
%   t4util_bdry             - Extract the boundary connectivity from a T4 connectivity.
%   t4util_coarsen          - Coarsen a T4 (tetrahedral) mesh
%   t4util_interfaces       - Extract the connectivity of the interface triangles
%   t4util_v_to_t4_map      - Vertex to tetrahedron map
%   t4util_v_to_t4e_map     - Vertex to edge map
%   t4util_vertex_neighbors - Find vertex neighbors
% 
% Voxel-image mesh generation:
%   voximg_load             - Load a Matlab file storing a voxel image.
%   voximg_to_H8            - Construct arrays to describe a hexahedron mesh created from voxel image.
%   voximg_to_T4            - Construct arrays to describe a tetrahedron mesh created from voxel image.
% 
% Miscellaneous meshes:
%   L2x2                    - 2 x 2 quadrilateral element mesh of one quarter of a square domain with a square hole.
%   bridge                  - Create mesh of a concrete bridge.
%   mpart                   - Mesh of a simple mechanical part
%   portal                  - Mesh of a truss portal.
% 
% Graphics utility:
%   drawmesh                - Draw a finite element mesh.
%   drawbasis2d             - Draw a basis function on a 2D finite element mesh.
% 
